#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <algorithm>
#include <vector>
#include <queue>
#include <map>
#include <set>
#include <cassert>
using namespace std;

#define FOR(i,a) for (__typeof((a).begin()) i = (a).begin(); i != (a).end(); ++ i)

const int maxn = 105;

int answer;
unsigned answerCnt;

inline void update(int &key, unsigned &cnt, int x, unsigned y)
{
    if (key < x) {
        key = x;
        cnt = y;
    } else if (key == x) {
        cnt += y;
    }
}

int bitmask[100][4], mask[100];

vector<vector<int> > blocks;
vector<int> cut;
vector<pair<int, int> > adj[maxn];

int dfn[maxn], low[maxn], stack[maxn], top, cnt, occur[maxn];
int n, m;

// my hash table for record states
namespace HashTable
{
    const int MOD = 10000007;

    struct Record {
        int state, conf, f, stamp; 
        unsigned g;
        // degree state, cant expanded nodes, maximum cost, count, time stamp
        Record() {
            stamp = -1;
            f = 0;
            g = 0;
        }
        Record(int state, int conf, int f, unsigned g, int stamp) : state(state), conf(conf), f(f), g(g), stamp(stamp) {
        }
    };
    
    Record record[MOD];
    int stamp;
    
    void update(int index, int f, unsigned g) {
        Record &r = record[index];
        if (r.f < f) {
            r.f = f;
            r.g = g;
        } else if (r.f == f) {
            r.g += g;
        }
    }
    
    void initialize() {
        stamp = 0;
    }
    
    void inc() {
        ++ stamp;
    }

    void insert(int state, int conf, int f, int g) {
        int h = ((conf << 20) + state) % MOD;
        while (record[h].stamp == stamp) {
            ++ h;
            if (h == MOD) {
                h = 0;
            }
        }
        record[h] = Record(state, conf, f, g, stamp);
    }
    
    int query(int state, int conf) {
        int h = ((conf << 20) + state) % MOD;
        while (record[h].stamp == stamp && (record[h].state != state || record[h].conf != conf)) {
            ++ h;
            if (h == MOD) {
                h = 0;
            }
        }
        if (record[h].stamp != stamp) {
            return -1;
        }
        return h;
    }
};

struct Edge
{
    int a, b, c;
    Edge(int a, int b, int c) : a(a), b(b), c(c) {
    }
};

// a bi-connect component, treat as a "tree" node
struct TreeNode
{
    TreeNode* parent;
    vector<TreeNode*> children;
    
    vector<int> treeEdge;
    
    void addTreeEdge(int i) {
        treeEdge.push_back(i);
    }
    
    vector<Edge> edge;
    
    priority_queue<pair<int,int> > heap[10];
    
    int maxState, *cmax;
    unsigned *cnt;
    bool *exists;
    
    int *rcmax[2];
    unsigned *rcnt[2];
    bool *rexists[2];
    
    vector<int> node;
    vector<vector<bool> > adj;
    vector<vector<int> > cost;
    
    set<int> nodeset;
    
    inline bool has(int x) {
        return nodeset.count(x);
    }
    
    ~TreeNode() {
        for (int i = 0; i < 2; ++ i) {
            delete [] rcmax[i];
            delete [] rcnt[i];
            delete [] rexists[i];
        }
    }
    
    TreeNode() {
        parent = NULL;
    }
    
    void addEdge(int a ,int b, int c) {
        edge.push_back(Edge(a, b, c));
    }
    
    void buildNodeFromEdge() {
        node.clear();
        for (int i = 0; i < edge.size(); ++ i) {
            node.push_back(edge[i].a);
            node.push_back(edge[i].b);
            nodeset.insert(edge[i].a);
            nodeset.insert(edge[i].b);
        }
        sort(node.begin(), node.end());
        node.erase(unique(node.begin(), node.end()), node.end());
    }
    
    inline void reorder() {
        vector<pair<int, int> > tmp;
        for (int i = 0; i < node.size(); ++ i) {
            if (parent != NULL && parent->has(node[i])) {
                tmp.push_back(make_pair(-1, node[i]));
            } else {
                int j;
                for (j = 0; j < children.size(); ++ j) {
                    if (children[j]->has(node[i])) {
                        break;
                    }
                }
                tmp.push_back(make_pair(j, node[i]));
            }
        }
        sort(tmp.begin(), tmp.end());
        map<int, int> id;
        for (int i = 0; i < node.size(); ++ i) {
            node[i] = tmp[i].second;
            id[node[i]] = i;
        }
        
        adj.assign(node.size(), vector<bool>(node.size(), false));
        cost.assign(node.size(), vector<int>(node.size(), 0));
        FOR (e, edge) {
            int a = id[e->a];
            int b = id[e->b];
            if (a != b) {
                adj[a][b] = adj[b][a] = true;
                cost[a][b] = cost[b][a] = e->c;
            }
        }
    }
    
    void generateState() {
        HashTable::inc();
        for (int i = 0, conf = 0; i < node.size(); ++ i) {
        //root with the smallest id i
            conf |= 1 << i; // cant expanded nodes
            for (int p = i + 1; p < node.size(); ++ p) {
                if (adj[i][p]) {
                    int state = bitmask[i][1] | bitmask[p][1];
                    heap[1].push(make_pair(state, -conf));
                    HashTable::insert(state, conf, cost[i][p], 1);
                    for (int q = p + 1; q < node.size(); ++ q) {
                        if (adj[i][q]) {
                            int state = bitmask[i][2] | bitmask[p][1] | bitmask[q][1];
                            heap[2].push(make_pair(state, -conf));
                            HashTable::insert(state, conf, cost[i][p] + cost[i][q], 1);
                            
                            for (int r = q + 1; r < node.size(); ++ r) {
                                if (adj[i][r]) {
                                    int state = bitmask[i][3] | bitmask[p][1] | bitmask[q][1] | bitmask[r][1];
                                    heap[3].push(make_pair(state, -conf));
                                    HashTable::insert(state, conf, cost[i][p] + cost[i][q] + cost[i][r], 1);
                                }
                            }
                        }
                    }
                }
            }
        }
        for (int sumEdge = 1; sumEdge < node.size(); ++ sumEdge) {
            while (heap[sumEdge].size()) {
                int state = heap[sumEdge].top().first;
                int conf = -heap[sumEdge].top().second;
                heap[sumEdge].pop();
                int index = HashTable::query(state, conf);
                int f = HashTable::record[index].f;
                int g = HashTable::record[index].g;
                
                int i = node.size(), pass = 0, k = node.size();
                for (int j = (int)node.size() - 1; j >= 0; -- j) {
                    if (state & bitmask[j][3]) {
                        ++ pass;
                        if (~conf >> j & 1) {
                            i = j;
                        }
                    } else if (conf >> j & 1) {
                        ++ pass;
                    } else {
                        k = j;
                    }
                }
                if (i == node.size()) {
                    // finish!
                    if (!exists[state] || f > cmax[state]) {
                        exists[state] = true;
                        cmax[state] = f;
                        cnt[state] = g;
                    } else if (f == cmax[state]) {
                        cnt[state] += g;
                    }
                } else {
                    // leaf i, to be extend
                    int newConf = conf | (1 << i);
                    
                    int degi = (state & bitmask[i][3]) / bitmask[i][1];
                    assert(degi == 1);
                    
                    if (true) {
                        //no extra edge
                        int newState = state;
                        int newF = f;
                        int index = HashTable::query(newState, newConf);
                        if (index < 0) {
                            HashTable::insert(newState, newConf, newF, g);
                            heap[sumEdge].push(make_pair(newState, -newConf));
                        } else {
                            HashTable::update(index, newF, g);
                        }
                    }
                    
                    for (int p = k; p < node.size(); ++ p) {
                        if ((state & bitmask[p][3]) == 0 && (conf >> p & 1) == 0 && adj[i][p]) {
                            //1 extra edge
                            if (degi + 1 <= 3) {
                                int newState = state + bitmask[i][1] + bitmask[p][1];
                                int newF = f + cost[i][p];
                                
                                int index = HashTable::query(newState, newConf);
                                if (index < 0) {
                                    HashTable::insert(newState, newConf, newF, g);
                                    heap[sumEdge + 1].push(make_pair(newState, -newConf));
                                } else {
                                    HashTable::update(index, newF, g);
                                }
                            }
                            
                            if (degi + 2 <= 3) {
                                for (int q = p + 1; q < node.size(); ++ q) {
                                    //2 extra edges
                                    if ((state & bitmask[q][3]) == 0 && (conf >> q & 1) == 0 && adj[i][q]) {
                                        int newState = state + bitmask[i][2] + bitmask[p][1] + bitmask[q][1];
                                        int newF = f + cost[i][p] + cost[i][q];
                                        int index = HashTable::query(newState, newConf);
                                        if (index < 0) {
                                            HashTable::insert(newState, newConf, newF, g);
                                            heap[sumEdge + 2].push(make_pair(newState, -newConf));
                                        } else {
                                            HashTable::update(index, newF, g);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    
    void local() {
        maxState = 1 << 2 * node.size();
        cmax = new int[maxState];
        cnt  = new unsigned[maxState];
        exists = new bool[maxState];
        memset(exists, false, maxState * sizeof(*exists));
        generateState();
       
        for (int i = 0; i < 2; ++ i) {
            rcmax[i] = new int[maxState];
            rcnt[i] = new unsigned[maxState];
            rexists[i] = new bool[maxState];
            memset(rexists[i], false, maxState * sizeof(*rexists[i]));
        }
        
        for (int s = 0; s < maxState; ++ s) {
            if (exists[s]) {
                int state = s;
                if (node.size() > 1) {
                    state &= mask[children.size() + 1];
                }
                if (state == 0) {
                    update(answer, answerCnt, cmax[s], cnt[s]);
                    continue;
                }
                if (!rexists[0][state]) {
                    rexists[0][state] = true;
                    rcmax[0][state] = cmax[s];
                    rcnt[0][state] = 0;
                }
                update(rcmax[0][state], rcnt[0][state], cmax[s], cnt[s]);
            }
        }
        
        if (node.size() == 1) {
            rexists[0][0] = 1;
            rcmax[0][0] = 0;
            rcnt[0][0] = 1;
        } else {
            maxState = 1 << 2 * (children.size() + 1);
        }
        
        delete [] cmax;
        delete [] cnt;
        delete [] exists;
        
        int now = 0, old = 1;
        for (int j = (int)children.size() - 1; j >= 0; -- j) {
            TreeNode *son = children[j];
            now = 1 - now;
            old = 1 - old;
            memset(rexists[now], false, maxState * sizeof(*rexists[now]));
            for (int s = 0; s < maxState; ++ s) {
                if (!rexists[old][s]) {
                    continue;
                }
                int drit = s;
                if (node.size() > 1) {
                    drit = (s >> (2 * (j + 1))) & 3;
                }
                for (int degree = 0; degree <= 3; ++ degree) {
                    if (son->rexists[0][degree]) {
                        if (node.size() > 1 && drit == 0 && degree != 0) {
                            continue;
                        }
                        int ds = drit + degree;
                        if (ds > 3) {
                            continue;
                        }
                        int state = ds;
                        if (node.size() > 1) {
                            state = s + bitmask[j + 1][degree];
                        }
                        int f = rcmax[old][s] + son->rcmax[0][degree];
                        unsigned g = rcnt[old][s] * son->rcnt[0][degree];
                        if (!rexists[now][state]) {
                            rexists[now][state] = true;
                            rcmax[now][state] = f;
                            rcnt[now][state] = 0;
                        }
                        update(rcmax[now][state], rcnt[now][state], f, g);
                    }
                }
            }
            
            now = 1 - now;
            old = 1 - old;
            memset(rexists[now], false, maxState * sizeof(*rexists[now]));
            for (int s = 0; s < maxState; ++ s) {
                if (!rexists[old][s]) {
                    continue;
                }
                int state = s;
                if (node.size() > 1) {
                    state &= mask[j + 1];
                }
                int f = rcmax[old][s];
                unsigned g = rcnt[old][s];
                if (state == 0) {
                    if (s == 0) {
                        continue;
                    }
                    update(answer, answerCnt, f, g);
                    continue;
                }
                if (!rexists[now][state]) {
                    rexists[now][state] = true;
                    rcmax[now][state] = f;
                    rcnt[now][state] = 0;
                }
                update(rcmax[now][state], rcnt[now][state], f, g);
            }
            if (node.size() == 1) {
                rexists[now][0] = 1;
                rcmax[now][0] = 0;
                rcnt[now][0] = 1;
            } else {
                maxState >>= 2;
            }
        }
        assert(now == 0);
        
        rexists[0][0] = 1;
		rcmax[0][0] = 0;
		rcnt[0][0] = 1;

		if (node.size() == 1) {
			for (int drit = 1; drit <= 3; drit++) {
				if (rexists[0][drit]) {
                    update(answer, answerCnt, rcmax[0][drit], rcnt[0][drit]);
				}
            }
		}
    }
};

// Tarjan, find the bi-connected components
inline void dfs(int u, int fa)
{
    dfn[u] = low[u] = ++ cnt;
    stack[++ top] = u;
    FOR (e, adj[u]) {
        int v = e->first;
        if (!dfn[v]) {
            int backup = top;
            dfs(v, u);
            low[u] = min(low[u], low[v]);
            if (low[v] >= dfn[u]) {
                blocks.push_back(vector<int>());
                for (int k = backup + 1; k <= top; ++ k) {
                    blocks.back().push_back(stack[k]);
                    ++ occur[stack[k]];
                }
                blocks.back().push_back(u);
                ++ occur[u];
                top = backup;
            }
        } else if (v != fa) {
            low[u] = min(low[u], dfn[v]);
        }
    }
}

bool visit[maxn * 2];
TreeNode *bicomp[2 * maxn];

inline void build(int u)
{
    visit[u] = true;

    FOR (v, bicomp[u]->treeEdge) {
        if (!visit[*v]) {
            bicomp[*v]->parent = bicomp[u];
            bicomp[u]->children.push_back(bicomp[*v]);
            build(*v);
        }
    }

    bicomp[u]->reorder();
}

inline void dp(TreeNode *u)
{
    FOR (v, u->children) {
        dp(*v);
    }
    u->local();
}

inline void solve()
{
    scanf("%d%d", &n, &m);
    for (int i = 0; i < n; ++ i) {
        adj[i].clear();
    }
    for (int i = 0; i < m; ++ i) {
        int a, b, c;
        scanf("%d%d%d", &a, &b, &c);
        -- a; -- b;
        adj[a].push_back(make_pair(b, c));
        adj[b].push_back(make_pair(a, c));
    }
    memset(dfn, 0, sizeof(dfn));
    memset(occur, 0, sizeof(occur));
    blocks.clear();
    top = cnt = 0;
    for (int i = 0; i < n; ++ i) {
        if (!dfn[i]) {
            dfs(i, -1);
        }
    }
//fprintf(stderr, "bicomp.size = %d\n", blocks.size());
    occur[0] = 2; // set node 0 to a cut node
    int cuts = 0, cutid[n];
    for (int i = 0; i < n; ++ i) {
        if (occur[i] > 1) {
            bicomp[blocks.size() + cuts] = new TreeNode();
            bicomp[blocks.size() + cuts]->addEdge(i, i, 0);
            bicomp[blocks.size() + cuts]->buildNodeFromEdge();
            cutid[i] = cuts;
            ++ cuts;
        }
    }
//fprintf(stderr, "cuts = %d\n", cuts);
    for (int i = 0; i < blocks.size(); ++ i) {
        bicomp[i] = new TreeNode();
        set<int> h(blocks[i].begin(), blocks[i].end());
        FOR (u, blocks[i]) {
            FOR (e, adj[*u]) {
                if (h.count(e->first)) {
                    bicomp[i]->addEdge(*u, e->first, e->second);
                }
            }
            if (occur[*u] > 1) {
                bicomp[i]->addTreeEdge(cutid[*u] + blocks.size());
                bicomp[cutid[*u] + blocks.size()]->addTreeEdge(i);
            }
        }
        bicomp[i]->buildNodeFromEdge();
    }
//fprintf(stderr, "build before\n");    
    memset(visit, false, sizeof(visit));
    build(blocks.size());
//fprintf(stderr, "build up\n");
    answer = 0; answerCnt = n + 1;
    dp(bicomp[blocks.size()]);
    
    for (int i = 0; i < blocks.size() + cuts; ++ i) {
        delete bicomp[i];
    }
    
    printf("%d %u\n", answer, answerCnt);
}

int main()
{
    for (int i = 0; i < 10; ++ i) {
        bitmask[i][0] = 0;
        for (int j = 1; j <= 3; ++ j) {
            bitmask[i][j] = j << 2 * i;
        }
        mask[i + 1] = (mask[i] << 2) | 3;
    }
    int T;
    for (scanf("%d", &T); T --; ) {
        solve();
    }
    return 0;
}
